"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getSignature = exports.sendToForDefi = void 0;
const fetch = require('node-fetch');
const fs_1 = require("fs");
const crypto_1 = __importDefault(require("crypto"));
const utils_1 = require("../src/utils");
const gatewayHost = "api.fordefi.com";
function sendToForDefi(unsignedTxidFile, ctxFile, withdrawal = false) {
    return __awaiter(this, void 0, void 0, function* () {
        const accessToken = (0, fs_1.readFileSync)("../token", 'utf8');
        const file = (0, fs_1.readFileSync)(ctxFile, 'utf8');
        const ctx = JSON.parse(file);
        const vault_id = ctx.vaultId;
        // vaultPublicKey should match public key in contex file
        let vaultPublicKey = yield getVaultPublickey(vault_id);
        if ((0, utils_1.unPrefix0x)(ctx.publicKey) != vaultPublicKey) {
            throw Error('public key does not match the vault');
        }
        let hash;
        let txidObj;
        if (!withdrawal) {
            txidObj = (0, utils_1.readUnsignedTxJson)(unsignedTxidFile);
            hash = txidObj.signatureRequests[0].message;
        }
        else {
            txidObj = (0, utils_1.readUnsignedWithdrawalTx)(unsignedTxidFile);
            hash = txidObj.message;
        }
        let hashBase64 = Buffer.from(hash, 'hex').toString('base64');
        const requestJson = {
            "vault_id": vault_id,
            "signer_type": "api_signer",
            "type": "black_box_signature",
            "details": {
                "format": "hash_binary",
                "hash_binary": hashBase64,
            },
        };
        const requestBody = JSON.stringify(requestJson);
        const path = "/api/v1/transactions";
        const timestamp = new Date().getTime();
        const payload = `${path}|${timestamp}|${requestBody}`;
        const privateKeyFile = "../private.pem";
        const secretPem = (0, fs_1.readFileSync)(privateKeyFile, 'utf8');
        const privateKey = crypto_1.default.createPrivateKey(secretPem);
        const sign = crypto_1.default.createSign('SHA256').update(payload, 'utf8').end();
        const signature1 = sign.sign(privateKey, 'base64');
        let response = yield fetch(`https://${gatewayHost}${path}`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                "Authorization": `Bearer ${accessToken}`,
                'X-Timestamp': timestamp,
                'X-Signature': signature1,
            },
            body: requestBody,
        });
        const responseJson = yield response.json();
        let txId = responseJson.id;
        // write tx id (to later fetch the signature)
        txidObj.forDefiTxId = txId;
        (0, fs_1.writeFileSync)(`${unsignedTxidFile}.unsignedTx.json`, JSON.stringify(txidObj), "utf8");
        return txId;
    });
}
exports.sendToForDefi = sendToForDefi;
function getSignature(unsignedTxidFile, withdrawal = false) {
    return __awaiter(this, void 0, void 0, function* () {
        const path = "/api/v1/transactions";
        const accessToken = (0, fs_1.readFileSync)("../token", 'utf8');
        let txidObj;
        let signedTxObj;
        if (!withdrawal) {
            txidObj = (0, utils_1.readUnsignedTxJson)(unsignedTxidFile);
            signedTxObj = txidObj;
        }
        else {
            txidObj = (0, utils_1.readUnsignedWithdrawalTx)(unsignedTxidFile);
            signedTxObj = txidObj;
        }
        let id = txidObj.forDefiTxId;
        let responseSignature;
        responseSignature = yield fetch(`https://${gatewayHost}${path}/${id}`, {
            method: 'GET',
            headers: {
                "Authorization": `Bearer ${accessToken}`,
            },
        });
        const responseJson = yield responseSignature.json();
        let signatureHex;
        try {
            signatureHex = Buffer.from(responseJson.signatures[0].data, 'base64').toString('hex');
        }
        catch (e) {
            throw Error("Transaction is not signed yet? " + e);
        }
        signedTxObj.signature = signatureHex;
        (0, fs_1.writeFileSync)(`${unsignedTxidFile}.signedTx.json`, JSON.stringify(signedTxObj), "utf8");
        return signatureHex;
    });
}
exports.getSignature = getSignature;
function getVaultPublickey(vaultId) {
    return __awaiter(this, void 0, void 0, function* () {
        const path = "/api/v1/vaults";
        const accessToken = (0, fs_1.readFileSync)("../token", 'utf8');
        let response = yield fetch(`https://${gatewayHost}${path}/${vaultId}`, {
            method: 'GET',
            headers: {
                "Authorization": `Bearer ${accessToken}`
            }
        });
        const responseJson = yield response.json();
        let pubKey = responseJson.public_key_compressed;
        let pubKeyHex = Buffer.from(pubKey, 'base64').toString('hex');
        return pubKeyHex;
    });
}
